searchData={"items":[{"type":"module","title":"YOLO","doc":"","ref":"YOLO.html"},{"type":"function","title":"YOLO.detect/2","doc":"","ref":"YOLO.html#detect/2"},{"type":"function","title":"YOLO.detect/3","doc":"","ref":"YOLO.html#detect/3"},{"type":"function","title":"YOLO.load/1","doc":"","ref":"YOLO.html#load/1"},{"type":"function","title":"YOLO.to_detected_objects/2","doc":"","ref":"YOLO.html#to_detected_objects/2"},{"type":"behaviour","title":"YOLO.FrameScaler","doc":"Behaviour module defining callbacks for part of the image preprocessing required by YOLO models.\n\nThe callbacks in this module handle three key preprocessing steps:\n\n1. Getting the input image dimensions to calculate proper scaling\n2. Resizing the image while preserving aspect ratio\n3. Converting the image to an Nx tensor in the correct format\n\nThe `YOLO.FrameScalers.fit/3` function uses the callbacks to:\n- Scale the image to fit the target dimensions while maintaining aspect ratio\n- Pad any remaining space with gray color (value 114).\n  [Why use 114,114,114?](https://github.com/ultralytics/ultralytics/issues/14584)\n\nImplementations of this behaviour provide the interface between different image\nprocessing libraries and the common YOLO preprocessing requirements.","ref":"YOLO.FrameScaler.html"},{"type":"callback","title":"YOLO.FrameScaler.image_resize/2","doc":"Resizes the input image according to the scaling configuration.\n\nThe scaling configuration contains the target dimensions and scaling factors needed to resize\nthe image while preserving aspect ratio. The resize is done in the implementation's native\nformat before later conversion to an Nx tensor via `image_to_nx/1`.","ref":"YOLO.FrameScaler.html#c:image_resize/2"},{"type":"callback","title":"Arguments - YOLO.FrameScaler.image_resize/2","doc":"* `image` - Input image in implementation's native format\n  * `scaling_config` - `ScalingConfig` struct containing target dimensions and scale factors","ref":"YOLO.FrameScaler.html#c:image_resize/2-arguments"},{"type":"callback","title":"Returns - YOLO.FrameScaler.image_resize/2","doc":"* Resized image in implementation's native format","ref":"YOLO.FrameScaler.html#c:image_resize/2-returns"},{"type":"callback","title":"YOLO.FrameScaler.image_shape/1","doc":"Returns the shape of the input image as a tuple of `{width, height, channels}`.\nThe shape information is used to properly scale and pad images while maintaining aspect ratio.","ref":"YOLO.FrameScaler.html#c:image_shape/1"},{"type":"callback","title":"YOLO.FrameScaler.image_to_nx/1","doc":"Converts the implementation's native image format to an Nx tensor.\nThe tensor should have shape `{height, width, channels}` where channels is typically 3 for RGB images.","ref":"YOLO.FrameScaler.html#c:image_to_nx/1"},{"type":"module","title":"YOLO.FrameScalers","doc":"Provides functions for resizing images while preserving aspect ratio and handling\npadding to reach target dimensions.","ref":"YOLO.FrameScalers.html"},{"type":"module","title":"Available scalers - YOLO.FrameScalers","doc":"The following scaler implementations are currently provided:\n\n* `YOLO.FrameScalers.EvisionScaler` - For scaling images using `Evision`\n* `YOLO.FrameScalers.ImageScaler` - For scaling images using `Image` library\n* `YOLO.FrameScalers.NxIdentityScaler` - For when input is already an appropriately sized `Nx` tensor\n\n```elixir\n# For Evision Mat images:\nYOLO.detect(model, mat, frame_scaler: YOLO.FrameScalers.EvisionScaler)\n\n# For Image library images:\nYOLO.detect(model, image, frame_scaler: YOLO.FrameScalers.ImageScaler)\n\n# For pre-sized Nx tensors:\nYOLO.detect(model, tensor, frame_scaler: YOLO.FrameScalers.NxIdentityScaler)\n```","ref":"YOLO.FrameScalers.html#module-available-scalers"},{"type":"function","title":"YOLO.FrameScalers.fit/3","doc":"Scales and pads an image to fit target dimensions.\nIt's used in the model `preprocess/3` callback.","ref":"YOLO.FrameScalers.html#fit/3"},{"type":"function","title":"Arguments - YOLO.FrameScalers.fit/3","doc":"* `image` - Input image in implementation's native format\n  * `target_shape` - Target dimensions as `{width, height}` tuple\n  * `resizer_module` - Module implementing the FrameScaler behaviour","ref":"YOLO.FrameScalers.html#fit/3-arguments"},{"type":"function","title":"Returns - YOLO.FrameScalers.fit/3","doc":"* `{image_tensor, scaling_config}` where image_tensor is the processed Nx tensor\n    and scaling_config is a struct containing scaling and padding information\n\nThe returned `image_tensor` has shape `{height, width, channels}`.","ref":"YOLO.FrameScalers.html#fit/3-returns"},{"type":"function","title":"YOLO.FrameScalers.scale_bboxes_to_original/2","doc":"Adjusts bounding box coordinates and sizes from the model input dimensions back to the original image dimensions.\n`bboxes` is a list of rows, where rows are lists of 6 elements `[cx, cy, w, h, prob, class_idx]`.\n\nReturns a list of scaled bboxes.\n\n# Example\nFor a YOLOv8n model processing an image resized to 640x640, use the following to scale\nthe bounding boxes back to the original 1920x1080 image:\n\n```\nscale_bboxes_to_original(bboxes, scaling_config)\n```","ref":"YOLO.FrameScalers.html#scale_bboxes_to_original/2"},{"type":"module","title":"YOLO.FrameScalers.EvisionScaler","doc":"","ref":"YOLO.FrameScalers.EvisionScaler.html"},{"type":"module","title":"YOLO.FrameScalers.ImageScaler","doc":"","ref":"YOLO.FrameScalers.ImageScaler.html"},{"type":"module","title":"YOLO.FrameScalers.NxIdentityScaler","doc":"`FrameScaler` for when the input image is already a tensor with shape {height, width, 3}\nand no scaling is needed because width and height already match the model input size.\nThis is useful when calling `YOLO.detect/3` with a tensor that already matches the\nexpected dimensions.","ref":"YOLO.FrameScalers.NxIdentityScaler.html"},{"type":"module","title":"YOLO.FrameScalers.ScalingConfig","doc":"Stores image scaling and padding configuration used during YOLO model preprocessing.","ref":"YOLO.FrameScalers.ScalingConfig.html"},{"type":"type","title":"YOLO.FrameScalers.ScalingConfig.t/0","doc":"","ref":"YOLO.FrameScalers.ScalingConfig.html#t:t/0"},{"type":"behaviour","title":"YOLO.Model","doc":"Defines a behaviour for implementing YOLO object detection models.\n\nThis module provides the structure for loading and running YOLO models for object detection.\nThe default implementation is `YOLO.Models.YoloV8`, but you can create custom implementations\nfor other YOLO variants.","ref":"YOLO.Model.html"},{"type":"behaviour","title":"Required Callbacks - YOLO.Model","doc":"To implement this behaviour, you need to define these functions:\n\n- `preprocess/3`: Prepares an input image for the model\n  - Takes a model struct, input image, and options\n  - Returns `{preprocessed_tensor, scaling_config}`\n  - See `YOLO.Models.YoloV8` for an example implementation\n\n- `postprocess/4`: Processes the model's raw output into detected objects\n  - Takes model struct, model output tensor, scaling config, and options\n  - Returns list of detected objects as `[cx, cy, w, h, prob, class_idx]`\n  - Handles tasks like non-maximum suppression and coordinate scaling","ref":"YOLO.Model.html#module-required-callbacks"},{"type":"behaviour","title":"Types - YOLO.Model","doc":"- `t()`: The model struct containing:\n  - `:ref` - Reference to loaded ONNX model\n  - `:model_impl` - Module implementing this behaviour\n  - `:shapes` - Input/output tensor shapes\n  - `:classes` - Map of class indices to labels\n\n- `detected_object()`: Map containing detection results:\n  - `:bbox` - Bounding box coordinates (cx, cy, w, h)\n  - `:class` - Detected class name\n  - `:class_idx` - Class index\n  - `:prob` - Detection probability","ref":"YOLO.Model.html#module-types"},{"type":"callback","title":"YOLO.Model.postprocess/4","doc":"Post-processes the model's raw output to produce a list of detected objects.\n\nThe raw output from the model is a tensor containing bounding box coordinates and class probabilities\nfor each candidate detection.\n\nFor example, YOLOv8 outputs a `{1, 84, 8400}` tensor where:\n  - 84 represents 4 bbox coordinates + 80 class probabilities\n  - 8400 represents the number of candidate detections\n\nReturns a list of detections where each detection is a list of 6 elements:\n```elixir\n[cx, cy, w, h, prob, class_idx]\n```\nwhere:\n- `cx`, `cy`: center x,y coordinates of bounding box\n- `w`, `h`: width and height of bounding box\n- `prob`: detection probability\n- `class_idx`: class index\n\nThe implementation should:\n1. Filter low probability detections\n2. Apply non-maximum suppression (NMS) to remove overlapping boxes\n3. Scale back the coordinates using the `scaling_config` and `YOLO.FrameScaler` module, since\n   the detections are based on the model's input resolution rather than the original image size\n\nSee `YOLO.Models.YoloV8.postprocess/4` for a reference implementation.","ref":"YOLO.Model.html#c:postprocess/4"},{"type":"callback","title":"YOLO.Model.preprocess/3","doc":"Prepares input image tensors for the model.","ref":"YOLO.Model.html#c:preprocess/3"},{"type":"callback","title":"Parameters - YOLO.Model.preprocess/3","doc":"* `model` - The YOLO.Model struct containing model information\n  * `image` - Input image in implementation's native format (e.g. Evision.Mat)\n  * `options` - Keyword list of options:\n    * `:frame_scaler` - Module implementing YOLO.FrameScaler behaviour (required)","ref":"YOLO.Model.html#c:preprocess/3-parameters"},{"type":"callback","title":"Returns - YOLO.Model.preprocess/3","doc":"* `{input_tensor, scaling_config}` tuple where:\n    * `input_tensor` is the preprocessed Nx tensor ready for model input, where shape is `{1, channels, height, width}`\n    * `scaling_config` contains scaling/padding info for postprocessing\n\nLook at the `YOLO.Models.YoloV8.preprocess/3` implementation to see how this callback is implemented.","ref":"YOLO.Model.html#c:preprocess/3-returns"},{"type":"type","title":"YOLO.Model.classes/0","doc":"","ref":"YOLO.Model.html#t:classes/0"},{"type":"type","title":"YOLO.Model.detected_object/0","doc":"","ref":"YOLO.Model.html#t:detected_object/0"},{"type":"type","title":"YOLO.Model.shape/0","doc":"","ref":"YOLO.Model.html#t:shape/0"},{"type":"type","title":"YOLO.Model.t/0","doc":"","ref":"YOLO.Model.html#t:t/0"},{"type":"module","title":"YOLO.Models","doc":"This module handles loading YOLO models and running object detection on images.\nThe default implementation supports YOLOv8 models, but the `YOLO.Model` behaviour can be\nimplemented for other YOLO variants.","ref":"YOLO.Models.html"},{"type":"module","title":"Main Functions - YOLO.Models","doc":"The key functions you'll use are:\n\n- `YOLO.Models.load/1`: Loads a YOLO model with required options\n  ```elixir\n  YOLO.Models.load(model_path: \"path/to/model.onnx\",\n                  classes_path: \"path/to/classes.json\",\n                  model_impl: YOLO.Models.YoloV8)\n  ```\n\n- `YOLO.Models.detect/3`: Runs object detection on an image\n  ```elixir\n  YOLO.Models.detect(model, image, prob_threshold: 0.5)\n  ```","ref":"YOLO.Models.html#module-main-functions"},{"type":"function","title":"YOLO.Models.detect/3","doc":"Performs object detection on an image using a loaded YOLO model.","ref":"YOLO.Models.html#detect/3"},{"type":"function","title":"Arguments - YOLO.Models.detect/3","doc":"* `model` - A loaded `YOLO.Model.t()` struct\n* `image` - Input image in the format expected by the frame scaler (e.g. `Evision.Mat`)\n* `opts` - Detection options","ref":"YOLO.Models.html#detect/3-arguments"},{"type":"function","title":"Options - YOLO.Models.detect/3","doc":"* `prob_threshold` - Minimum probability threshold for detections (default: `0.25`)\n* `iou_threshold` - IoU threshold for non-maximum suppression (default: `0.45`)\n* `frame_scaler` - Module implementing `YOLO.FrameScaler` behaviour (default: `YOLO.FrameScalers.EvisionScaler`)","ref":"YOLO.Models.html#detect/3-options"},{"type":"function","title":"Returns - YOLO.Models.detect/3","doc":"A list of detections, where each detection is a list `[cx, cy, w, h, prob, class_idx]`:\n* `cx`, `cy` - Center coordinates of bounding box\n* `w`, `h` - Width and height of bounding box\n* `prob` - Detection probability\n* `class_idx` - Class index\n\nThe output can be converted to structured maps using `to_detected_objects/1`.","ref":"YOLO.Models.html#detect/3-returns"},{"type":"function","title":"Example - YOLO.Models.detect/3","doc":"model\n    |> YOLO.Model.detect(image, prob_threshold: 0.5)\n    |> YOLO.Model.to_detected_objects()","ref":"YOLO.Models.html#detect/3-example"},{"type":"function","title":"YOLO.Models.load/1","doc":"Loads a YOLO model from an ONNX file.","ref":"YOLO.Models.html#load/1"},{"type":"function","title":"Required Options - YOLO.Models.load/1","doc":"* `model_path` - Path to the `.onnx` model file\n* `classes_path` - Path to the `.json` file containing class labels","ref":"YOLO.Models.html#load/1-required-options"},{"type":"function","title":"Optional Options - YOLO.Models.load/1","doc":"* `model_impl` - Module implementing the `YOLO.Model` behaviour (default: `YOLO.Models.YoloV8`)\n* `json_decoder` - Function to decode JSON strings (default: `&:json.decode/1`)","ref":"YOLO.Models.html#load/1-optional-options"},{"type":"function","title":"Returns - YOLO.Models.load/1","doc":"A `YOLO.Model.t()` struct containing:\n* `ref` - Reference to the loaded ONNX model\n* `model_impl` - The module implementing the model version\n* `classes` - Map of class indices to labels\n* `shapes` - Input/output tensor shapes","ref":"YOLO.Models.html#load/1-returns"},{"type":"function","title":"Example - YOLO.Models.load/1","doc":"```elixir\n  YOLO.Model.load(\n    model_path: \"models/yolov8n.onnx\",\n    classes_path: \"models/coco_classes.json\"\n  )\n  ```","ref":"YOLO.Models.html#load/1-example"},{"type":"function","title":"YOLO.Models.run/2","doc":"Runs inference on a preprocessed input tensor.","ref":"YOLO.Models.html#run/2"},{"type":"function","title":"Arguments - YOLO.Models.run/2","doc":"* `model` - A loaded `YOLO.Model.t()` struct\n* `image_tensor` - Preprocessed input tensor matching model's expected shape","ref":"YOLO.Models.html#run/2-arguments"},{"type":"function","title":"Returns - YOLO.Models.run/2","doc":"The raw output tensor from the model. For YOLOv8n:\n* Input shape: `{1, 3, 640, 640}`\n* Output shape: `{1, 84, 8400}`\n\nThis is typically used internally by `detect/3` and shouldn't need to be called directly.","ref":"YOLO.Models.html#run/2-returns"},{"type":"module","title":"YOLO.Models.YoloV8","doc":"YOLOv8 model implementation for preprocessing input images\nand postprocessing detections using non-maximum suppression (NMS).","ref":"YOLO.Models.YoloV8.html"},{"type":"function","title":"YOLO.Models.YoloV8.postprocess/4","doc":"Post-processes the model's raw output to produce a filtered list of detected objects.\n\nThe raw output tensor has shape {1, 84, 8400} where:\n- First dimension (1) is the batch size\n- Second dimension (84) contains 4 bbox coordinates + 80 class probabilities per detection\n- Third dimension (8400) is the number of candidate detections\n\nThe processing steps are:\n1. Reshapes and transposes output to {8400, 84} format\n2. Applies non-maximum suppression (NMS) to filter overlapping detections\n3. Scales bounding boxes back to original image dimensions","ref":"YOLO.Models.YoloV8.html#postprocess/4"},{"type":"function","title":"Arguments - YOLO.Models.YoloV8.postprocess/4","doc":"* `model` - YOLO.Model struct containing model metadata\n  * `model_output` - Raw output tensor from model inference `{1, 84, 8400}`\n  * `scaling_config` - Scaling configuration from preprocessing step\n  * `opts` - Keyword list of options:\n    * `prob_threshold` - Minimum probability threshold for detections\n    * `iou_threshold` - IoU threshold for non-maximum suppression\n    * `nms_fun` - Optional custom NMS function (defaults to `YOLO.NMS.run/3`)","ref":"YOLO.Models.YoloV8.html#postprocess/4-arguments"},{"type":"function","title":"Returns - YOLO.Models.YoloV8.postprocess/4","doc":"List of detections where each detection is a list [cx, cy, w, h, prob, class_idx]:\n  * cx, cy - Center coordinates of bounding box\n  * w, h - Width and height of bounding box\n  * prob - Detection probability\n  * class_idx - Class index","ref":"YOLO.Models.YoloV8.html#postprocess/4-returns"},{"type":"function","title":"YOLO.Models.YoloV8.preprocess/3","doc":"Preprocesses an input image to match the model's required format.\n\nThe preprocessing steps are:\n1. Scales and pads the image to match model input dimensions while preserving aspect ratio\n2. Converts RGB to BGR color format (YOLOv8 models expect BGR input)\n3. Normalizes pixel values to [0,1] range by dividing by 255\n4. Transposes dimensions to match model's expected format\n5. Adds batch dimension","ref":"YOLO.Models.YoloV8.html#preprocess/3"},{"type":"function","title":"Arguments - YOLO.Models.YoloV8.preprocess/3","doc":"* `model` - YOLO.Model struct containing model metadata\n  * `image` - Input image in implementation's native format\n  * `options` - Keyword list of options including `:frame_scaler` module","ref":"YOLO.Models.YoloV8.html#preprocess/3-arguments"},{"type":"function","title":"Returns - YOLO.Models.YoloV8.preprocess/3","doc":"* `{input_tensor, scaling_config}` tuple where:\n    * `input_tensor` has shape {1, 3, height, width}\n    * `scaling_config` contains scaling/padding info for postprocessing","ref":"YOLO.Models.YoloV8.html#preprocess/3-returns"},{"type":"module","title":"YOLO.NMS","doc":"Elixir NMS (Non-Maximum Suppression)\n\nLearn more about Non-Maximum Suppression (NMS) at:\nhttps://builtin.com/machine-learning/non-maximum-suppression\n\nThis implementation applies NMS independently for each output class.\nThe following steps are executed for each class:\n1. Filter out all bounding boxes with maximum class probability below `prob_threshold` (default: `0.5`).\n2. Select the bounding box with the highest `prob`.\n3. Remove any remaining bounding boxes with an IoU >= `iou_threshold` (default: `0.5`).","ref":"YOLO.NMS.html"},{"type":"function","title":"YOLO.NMS.filter_predictions/2","doc":"Filters detections, keeping only those with a probability higher than `:prob_threshold`.\n\nThe input `tensor` must have the shape `{8400, 84}` (transposed YOLOv8 output format).\n\nReturns a list of `[bbox_cx, bbox_cy, bbox_w, bbox_h, prob, class_idx]`.\n\nThis implementation is inspired by Hans Elias B. Josephsen's talk (see the filter function at 12:06):\nhttps://youtu.be/OsxGB6MbA8o?t=726","ref":"YOLO.NMS.html#filter_predictions/2"},{"type":"function","title":"YOLO.NMS.iou/2","doc":"IoU (Intersection over Union) measures the ratio of the area of overlap between\ntwo bounding boxes to the area of their union.\n\nIoU = A ∩ B / A U B","ref":"YOLO.NMS.html#iou/2"},{"type":"function","title":"YOLO.NMS.nms/2","doc":"Applies Non-Maximum Suppression (NMS) for each class, discarding bounding boxes with an\nIoU exceeding the specified `iou_threshold`.","ref":"YOLO.NMS.html#nms/2"},{"type":"function","title":"YOLO.NMS.run/3","doc":"Runs both `filter_predictions/2` and `nms/2`.\n\n1. Filters out detections with a probability below `prob_threshold` (`p < prob_threshold`)\n2. Applies Non-Maximum Suppression (NMS) for each class, discarding bounding boxes with an\n   IoU exceeding the specified `iou_threshold`.\n\nThe input `tensor` must have the shape `{8400, 84}` (transposed YOLOv8 output format).\n\nReturns a list of `[bbox_cx, bbox_cy, bbox_w, bbox_h, prob, class_idx]`.","ref":"YOLO.NMS.html#run/3"},{"type":"module","title":"YOLO.Utils","doc":"Utils to transform the model input and output.","ref":"YOLO.Utils.html"},{"type":"function","title":"YOLO.Utils.to_detected_objects/2","doc":"Maps `[cx, cy, w, h, prob, class_idx]` rows to `YOLO.Model.detected_object()`\n\n# Example\n```\niex> YOLO.Utils.to_detected_objects([ [100, 200, 20, 30, 0.7, 2] ], yolov8n_model.classes)\n[%{\n  bbox: %{cx: 100, cy: 200, w: 20, h: 30},\n  prob: 0.7,\n  class_idx: 2,\n  class: \"car\"\n}]\n```","ref":"YOLO.Utils.html#to_detected_objects/2"},{"type":"extras","title":"README","doc":"You Only Look Once (YOLO) is a cutting-edge, real-time object detection system. It works by predicting bounding boxes and class probabilities directly from an input image in a single evaluation, making it exceptionally fast compared to other object detection methods. YOLO models are widely used in applications like autonomous driving, surveillance, and robotics due to their balance of speed and accuracy.  \n\n![traffic image](guides/images/traffic_detections.jpg)\n\n\n\n`YOLO` is an Elixir library designed to simplify object detection by providing seamless integration of YOLO models. With this library, you can efficiently utilize the power of YOLO for real-time object detection.  \n\nThe library is built with the following objectives:  \n\n1. **Speed**  \n   Achieve real-time object detection performance, even on resource-constrained devices like a MacBook Air M3. Details about performance can be found in the [benchmarks section](#benchmarks).  \n\n2. **Ease of Use**  \n   Designed to be straightforward and user-friendly, the library enables developers to load and detect objects with just two function calls.\n\n3. **Extensibility**  \n   Currently supporting YOLOv8 (in various model sizes such as `n`, `s`, ..., `x`), the library is built around a `YOLO.Model` behavior. It leverages ONNX and Ortex, making it adaptable for supporting other YOLO versions or custom models in the future.","ref":"readme.html"},{"type":"extras","title":"IMPORTANT: ONNX model files! - README","doc":"To run a YOLOv8 model, we typically need the neural network structure and the pre-trained weights obtained from training the model. While we could build the network using libraries like Axon (or Keras in Python), I've prefered to simplify the process by using a pre-existing PyTorch model from [Ultralytics](https://docs.ultralytics.com/models/yolov8/). Ultralytics developed and trained [YOLOv8 models](https://github.com/ultralytics/assets/releases/tag/v8.3.0), which are available under a GPL license. However, to avoid potential licensing conflicts with this library, I haven't included the ONNX model exports directly here. Instead, you need to convert the PyTorch .pt file into an ONNX format. No worries, I got you covered!\n\nUltralytics' YOLOv8 comes in several sizes: n (nano), s (small), m (medium), l (large), and x (extra-large). Larger models offer better performance in terms of classification accuracy and object detection but require more computational resources and memory.\n\n\nYou can use the `python/yolov8_to_onnx.py` script found in the [GitHub repo](https://github.com/poeticoding/yolo_elixir).\n\nFirst, install the dependencies (`requests` and `ultralytics`)\n```bash\npip install -r python/requirements.txt\n```\n\nThen, run the script by specifying the model size, such as `n`:\n\n```bash\npython python/yolov8_to_onnx.py n\n```\n\n\nThe script will download the `.pt` model and generate two files:\n\n* `models/yolov8n.onnx`: the YOLOv8n model with weights\n* `models/yolov8n_classes.json`: the list of object classes","ref":"readme.html#important-onnx-model-files"},{"type":"extras","title":"Getting Started - README","doc":"First install the library and configure Nx.\n```elixir\nMix.install([\n  {:yolo, \"~> 0.1.0\"},\n\n  # I'm using EXLA as Nx backend on my MacBook Air M3\n  {:exla, \"~> 0.9.2\"},\n  # evision for image processing (you can use :image instead)\n  {:evision, \"~> 0.2.0\"}\n]\n], config: [\n  nx: [default_backend: EXLA.Backend]\n])\n```\n\nThen you need just a few lines of code to get a list of objects detected in the image.\n\n```elixir  \nmodel = YOLO.load([\n  model_path: \"models/yolov8n.onnx\", \n  classes_path: \"models/yolov8n_classes.json\"\n])\n\nmat = Evision.imread(image_path)\n\nmodel\n|> YOLO.detect(mat)\n|> YOLO.to_detected_objects(model.classes)\n\n```\nThis code returns a list of detected objects along with their bounding box coordinates.\n\n```elixir\n[\n  %{\n    class: \"person\",\n    prob: 0.5744523406028748,\n    bbox: %{h: 126, w: 70, cx: 700, cy: 570},\n    class_idx: 0\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.6159384846687317,\n    bbox: %{h: 102, w: 71, cx: 726, cy: 738},\n    class_idx: 1\n  },\n  %{class: \"car\", prob: 0.6243442893028259, bbox: %{h: 87, w: 102, cx: 1039, cy: 268}, class_idx: 2},\n  ...\n]\n```\n\n![traffic image](guides/images/traffic_detections.jpg)","ref":"readme.html#getting-started"},{"type":"extras","title":"YoloV8x - README","doc":"To run a larger YoloV8 model is simple.\n\nUse the python script to generate the `yolov8x.onnx` model and the `yolov8x_classes.json` file.\n\n```\npython python/yolov8_to_onnx.py x\n```\n\nThen you can load the model and use it just like before.\n\n```elixir\nmodel_x = YOLO.load(model_path: \"code/yolo/models/yolov8x.onnx\", classes_path: classes_path)\n\nmodel_x\n|> YOLO.detect(mat)\n|> YOLO.to_detected_objects(model_x.classes)    \n\n```\n![traffic image](guides/images/traffic_yolov8x_detections.jpg)","ref":"readme.html#yolov8x"},{"type":"extras","title":"Make it faster with FastNMS - README","doc":"Non-Maximum Suppression (NMS) is what makes the postprocessing slow in the Elixir implementation. The native Nx implementation is not fast enough to achieve good real-time performance. With the `YoloFastNMS` library providing a Rust NIF implementation, it's possible to run the postprocessing much faster - simply include the dependency and pass the module to `detect/3`. This can **speed up postprocessing by around 100x** (from ~400ms to ~4ms on a MacBook Air M3).\n\n```elixir\ndef deps do\n  [\n    {:yolo_fast_nms, \"~> 0.1\"}\n  ]\nend\n```\n\nThen you can run the detection with FastNMS by passing the `YoloFastNMS` module to `detect/3`.\n```elixir\nYOLO.detect(model, mat, nms_fun: &YoloFastNMS.run/3)\n```","ref":"readme.html#make-it-faster-with-fastnms"},{"type":"extras","title":"Benchmarks - README","doc":"Benchmarks below are run on a MacBook Air M3 with EXLA as Nx backend.\n\n```text\nOperating System: macOS\nCPU Information: Apple M3\nNumber of Available Cores: 8\nAvailable memory: 16 GB\nElixir 1.17.2\nErlang 27.1.2\nJIT enabled: true\n```\n### `detect/3` with `YoloFastNMS` vs Ultralytics \n[benchmarks/yolov8n.exs](benchmarks/yolov8n.exs)\n```text\nName                            ips        average  deviation         median         99th %\ndetect/3 with FastNMS         26.09       38.32 ms     ±4.43%       38.09 ms       48.83 ms\n\nMemory usage statistics:\n\nName                          average  deviation         median         99th %\ndetect/3 with FastNMS        48.42 KB     ±5.92%       47.25 KB       56.06 KB\n```\n\n[benchmarks/yolov8n_cpu.py](benchmarks/yolov8n_cpu.py)\n\n```text\n0: 384x640 17 persons, 3 bicycles, 6 cars, 1 truck, 2 traffic lights, 39.0ms\nSpeed: 1.2ms preprocess, 39.0ms inference, 0.3ms postprocess per image at shape (1, 3, 384, 640)\n```\n\n\n\n### `YOLO.NMS` vs `YoloFastNMS`\n[benchmarks/nms.exs](benchmarks/nms.exs)\n```text\nName                  ips        average  deviation         median         99th %\nYoloFastNMS        528.63        1.89 ms     ±0.56%        1.89 ms        1.93 ms\nYOLO.NMS             2.28      437.78 ms     ±0.43%      438.21 ms      441.01 ms\n\nComparison:\nYoloFastNMS        528.63\nYOLO.NMS             2.28 - 231.42x slower +435.89 ms\n\nMemory usage statistics:\n\nName           Memory usage\nYoloFastNMS      0.00651 MB\nYOLO.NMS          810.22 MB - 124497.68x memory usage +810.21 MB\n```\n\n### `preprocess/3`, `run/2`, `postprocess/4`\n[benchmarks/yolov8n_pipeline.exs](benchmarks/yolov8n_pipeline.exs)\n```text\nName                          ips        average  deviation         median         99th %\npreprocess                 467.31        2.14 ms    ±11.15%        2.12 ms        2.76 ms\npostprocess FastNMS        421.43        2.37 ms     ±6.04%        2.36 ms        2.68 ms\nrun                         27.04       36.98 ms     ±3.01%       36.74 ms       40.18 ms\n\nComparison:\npreprocess                 467.31\npostprocess FastNMS        421.43 - 1.11x slower +0.23 ms\nrun                         27.04 - 17.28x slower +34.84 ms\n\nMemory usage statistics:\n\nName                        average  deviation         median         99th %\npreprocess                 43.78 KB     ±0.00%       43.78 KB       43.78 KB\npostprocess FastNMS         2.02 KB    ±92.71%        1.18 KB        6.62 KB\nrun                         1.18 KB     ±0.00%        1.18 KB        1.18 KB\n\nComparison:\npreprocess                 43.78 KB\npostprocess FastNMS         2.02 KB - 0.05x memory usage -41.76387 KB\nrun                         1.18 KB - 0.03x memory usage -42.60156 KB\n```","ref":"readme.html#benchmarks"},{"type":"extras","title":"Livebook Examples - README","doc":"* [YoloV8 Single Image](examples/yolov8_single_image.livemd): A beginner-friendly example that demonstrates object detection on a single image, comparing results between the lightweight YOLOv8n model and the more accurate YOLOv8x model.\n* [Make it faster with YoloFastNMS](examples/yolo_fast_nms.livemd): A livebook that shows how to use the `YoloFastNMS` library to speed up the object detection postprocessing.\n* [Real-time Object Detection](examples/yolov8_webcam.livemd): A livebook that demonstrates real-time object detection using your computer's webcam, with live frame updates and visualization of detected objects.","ref":"readme.html#livebook-examples"},{"type":"extras","title":"Under the Hood - README","doc":"Let's see how `YOLO.detect/3` works.","ref":"readme.html#under-the-hood"},{"type":"extras","title":"Load YoloV8n Model - README","doc":"Loads the *YoloV8n* model using the `model_path` and `classes_path`. Optionally, specify `model_impl`, which defaults to `YOLO.Models.YoloV8`.\n\n```elixir\nmodel = YOLO.load([\n  model_path: \"models/yolov8n.onnx\", \n  classes_path: \"models/yolov8n_classes.json\"\n])\n```","ref":"readme.html#load-yolov8n-model"},{"type":"extras","title":"Preprocessing - README","doc":"```elixir\nmat = Evision.imread(image_path)\n\n{input_tensor, scaling_config} = YOLO.Models.YoloV8.preprocess(model, mat, [frame_scaler: YOLO.FrameScalers.EvisionScaler])\n```\n\nBefore running object detection, the input image needs to be preprocessed to match the model's expected input format. The preprocessing steps are:\n\n1. **Resize and Pad Image to 640x640**\n   - The image is resized while preserving aspect ratio to fit within 640x640 pixels\n   - Any remaining space is padded with gray color (value 114) to reach exactly 640x640\n   - This is handled by the `FrameScaler` behaviour and its implementations\n\n2. **Convert to Normalized Tensor**\n   - The image is converted to an Nx tensor with shape `{1, 3, 640, 640}`\n   - Pixel values are normalized from `0-255` to `0.0-1.0` range\n   - The channels are reordered from `RGB` to the model's expected format (`BGR` in this case)\n\nThe `FrameScaler` behaviour provides a consistent interface for handling different image formats:\n\n- `EvisionScaler` - For OpenCV Mat images from Evision\n- `ImageScaler` - For images using the Image library  \n- `NxIdentityScaler` - For ready to use Nx tensors","ref":"readme.html#preprocessing"},{"type":"extras","title":"Run Object Detection - README","doc":"Then run the detection by passing the `model` and the image tensor `input_tensor`.\n\n```elixir\n# input_tensor {1, 3, 640, 640}\noutput_tensor = YOLO.Models.run(model, input_tensor)\n# output_tensor {1, 84, 8400}\n```\n\nYou can also adjust detection thresholds (`iou_threshold` and `prob_threshold`, which both default to `0.45` and `0.25` respectively) using the third argument.","ref":"readme.html#run-object-detection"},{"type":"extras","title":"Postprocessing - README","doc":"```elixir\nresult_rows = YOLO.Models.YoloV8.postprocess(model, output_tensor, scaling_config, opts)\n```\nwhere `result_rows` is a list of lists, where each inner list represents a detected object with 6 elements:\n\n```elixir\n[\n  [cx, cy, w, h, prob, class_idx],\n  ...\n]\n```\n\nThe model's raw output needs to be post-processed to extract meaningful detections. For YOLOv8n, the `output_tensor` has shape `{1, 84, 8400}` where:\n\n- 84 represents 4 bbox coordinates + 80 class probabilities\n- 8400 represents the number of candidate detections\n\nThe postprocessing steps are:\n\n1. **Filter Low Probability Detections**\n   - Each of the 8400 detections has probabilities for 80 classes\n   - Only keep detections where max class probability exceeds `prob_threshold` (default 0.25)\n\n2. **Non-Maximum Suppression (NMS)**\n   - Remove overlapping boxes for the same object\n   - For each class, compare boxes using Intersection over Union (IoU)\n   - If IoU > `iou_threshold` (default 0.45), keep only highest probability box\n   - This prevents multiple detections of the same object\n\n3. **Scale Coordinates**\n   - The detected coordinates are based on the model's 640x640 input\n   - Use the `scaling_config` from preprocessing to map back to original image size\n   - This accounts for any resizing/padding done during preprocessing","ref":"readme.html#postprocessing"},{"type":"extras","title":"Convert Detections to Structured Maps - README","doc":"Finally, convert the raw detection results into structured maps containing bounding box coordinates, class labels, and probabilities:\n```elixir\niex> YOLO.to_detected_objects(result_rows, model.classes)\n[\n  %{\n    class: \"person\",\n    prob: 0.57,\n    bbox: %{h: 126, w: 70, cx: 700, cy: 570},\n    class_idx: 0\n  },\n  ...\n]\n```","ref":"readme.html#convert-detections-to-structured-maps"},{"type":"extras","title":"Current Limitations - README","doc":"The current implementation supports YOLOv8 models with a fixed `640x640` input size (even though YOLOv8x6 supports `1280x1280` images) and a fixed `84x8400` output size. This setup handles 80 classes from the COCO dataset and 8400 detections.\n\nThe library is designed to be extensible through the `YOLO.Model` behaviour, allowing other YOLO versions or custom model implementations to be added in the future.","ref":"readme.html#current-limitations"},{"type":"extras","title":"Future Plans - README","doc":"One of the next goals (listed in the TODO section below) is to support models with different input and output sizes. This update would allow the library to work with YOLO models trained on other datasets or even custom datasets, making it more flexible and useful.","ref":"readme.html#future-plans"},{"type":"extras","title":"TODOs - README","doc":"","ref":"readme.html#todos"},{"type":"extras","title":"Improvements - README","doc":"* [ ] Support dynamically different input and output shapes. (This is going to be foundamental to support different models with custom classes).\n* [ ] Kino library to easily visualize detections","ref":"readme.html#improvements"},{"type":"extras","title":"Experiments - README","doc":"* [ ] CUDA benchmarks\n* [ ] Object tracking\n* [ ] Run it on Nerves Rpi5\n* [ ] Run it on Jetson\n* [ ] Sharding","ref":"readme.html#experiments"},{"type":"extras","title":"YoloV8 on a single image","doc":"<!-- livebook:{\"file_entries\":[{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/test/fixtures/traffic.jpg\"},\"name\":\"traffic.jpg\",\"type\":\"file\"},{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/models/yolov8n.onnx\"},\"name\":\"yolov8n.onnx\",\"type\":\"file\"},{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/models/yolov8n_classes.json\"},\"name\":\"yolov8n_classes.json\",\"type\":\"file\"},{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/models/yolov8x.onnx\"},\"name\":\"yolov8x.onnx\",\"type\":\"file\"},{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/models/yolov8x_classes.json\"},\"name\":\"yolov8x_classes.json\",\"type\":\"file\"}],\"persist_outputs\":true} -->\n\n# YoloV8 on a single image\n\n```elixir\nMix.install([\n  {:yolo, \">= 0.0.0\"},\n  {:nx, \"~> 0.9.1\"},\n  {:exla, \"~> 0.9.0\"},\n  {:kino, \"~> 0.14.0\"},\n  {:image, \"~> 0.54\"},\n  {:evision, \"~> 0.2\"}\n],[\n  config: [\n    nx: [default_backend: EXLA.Backend]\n  ]\n])\n```","ref":"yolov8_single_image.html"},{"type":"extras","title":"Define Paths - YoloV8 on a single image","doc":"```elixir\nimage_path = Kino.FS.file_path(\"traffic.jpg\")\nmodel_path = Kino.FS.file_path(\"yolov8n.onnx\")\nclasses_path = Kino.FS.file_path(\"yolov8n_classes.json\")\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"yolov8_single_image.html#define-paths"},{"type":"extras","title":"Load image - YoloV8 on a single image","doc":"```elixir\nmat = Evision.imread(image_path)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Evision.Mat{\n  channels: 3,\n  dims: 2,\n  type: {:u, 8},\n  raw_type: 16,\n  shape: {1080, 1920, 3},\n  ref: #Reference<0.133361986.436862999.157550>\n}\n```","ref":"yolov8_single_image.html#load-image"},{"type":"extras","title":"Load and run YoloV8n - YoloV8 on a single image","doc":"```elixir\nmodel = YOLO.load(model_path: model_path, classes_path: classes_path)\n\ndetected_objects = \n  model\n  |> YOLO.detect(mat, iou_threshold: 0.45, prob_threshold: 0.25)\n  |> YOLO.to_detected_objects(model.classes)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  %{\n    class: \"person\",\n    prob: 0.2571249008178711,\n    bbox: %{h: 237, w: 22, cx: 1908, cy: 468},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.3452834486961365,\n    bbox: %{h: 119, w: 45, cx: 697, cy: 396},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.3655376434326172,\n    bbox: %{h: 213, w: 98, cx: 674, cy: 862},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.37552228569984436,\n    bbox: %{h: 79, w: 33, cx: 867, cy: 208},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.4229397177696228,\n    bbox: %{h: 108, w: 38, cx: 1500, cy: 364},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.43063610792160034,\n    bbox: %{h: 159, w: 55, cx: 1770, cy: 501},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.4697304964065552,\n    bbox: %{h: 117, w: 95, cx: 730, cy: 651},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.5778378248214722,\n    bbox: %{h: 117, w: 46, cx: 556, cy: 403},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.6154893040657043,\n    bbox: %{h: 150, w: 72, cx: 47, cy: 525},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.6168261766433716,\n    bbox: %{h: 161, w: 53, cx: 800, cy: 579},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.6266347169876099,\n    bbox: %{h: 136, w: 44, cx: 1614, cy: 419},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.6296634078025818,\n    bbox: %{h: 156, w: 49, cx: 1686, cy: 489},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.655760645866394,\n    bbox: %{h: 113, w: 40, cx: 1531, cy: 416},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.6911468505859375,\n    bbox: %{h: 237, w: 79, cx: 39, cy: 799},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.7227481603622437,\n    bbox: %{h: 152, w: 59, cx: 298, cy: 557},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.742075502872467,\n    bbox: %{h: 225, w: 89, cx: 468, cy: 848},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.7557895183563232,\n    bbox: %{h: 214, w: 80, cx: 609, cy: 776},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.7682790756225586,\n    bbox: %{h: 146, w: 69, cx: 699, cy: 578},\n    class_idx: 0\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.3721795678138733,\n    bbox: %{h: 123, w: 63, cx: 461, cy: 977},\n    class_idx: 1\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.47977668046951294,\n    bbox: %{h: 106, w: 71, cx: 723, cy: 738},\n    class_idx: 1\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.500577449798584,\n    bbox: %{h: 127, w: 56, cx: 590, cy: 862},\n    class_idx: 1\n  },\n  %{class: \"car\", prob: 0.35978999733924866, bbox: %{h: 66, w: 73, cx: 1169, cy: 145}, class_idx: 2},\n  %{class: \"car\", prob: 0.39499765634536743, bbox: %{h: 84, w: 93, cx: 1028, cy: 211}, class_idx: 2},\n  %{\n    class: \"car\",\n    prob: 0.42516815662384033,\n    bbox: %{h: 81, w: 100, cx: 1203, cy: 243},\n    class_idx: 2\n  },\n  %{class: \"car\", prob: 0.6461262106895447, bbox: %{h: 91, w: 102, cx: 1037, cy: 268}, class_idx: 2},\n  %{\n    class: \"car\",\n    prob: 0.6885334253311157,\n    bbox: %{h: 108, w: 124, cx: 1251, cy: 326},\n    class_idx: 2\n  },\n  %{\n    class: \"car\",\n    prob: 0.6895469427108765,\n    bbox: %{h: 114, w: 141, cx: 1305, cy: 403},\n    class_idx: 2\n  },\n  %{\n    class: \"truck\",\n    prob: 0.4962051510810852,\n    bbox: %{h: 246, w: 191, cx: 1100, cy: 473},\n    class_idx: 7\n  },\n  %{\n    class: \"traffic light\",\n    prob: 0.3155118525028229,\n    bbox: %{h: 100, w: 49, cx: 194, cy: 398},\n    class_idx: 9\n  },\n  %{\n    class: \"traffic light\",\n    prob: 0.4702877998352051,\n    bbox: %{h: 140, w: 76, cx: 1334, cy: 103},\n    class_idx: 9\n  }\n]\n```","ref":"yolov8_single_image.html#load-and-run-yolov8n"},{"type":"extras","title":"Showing Detected Objects - YoloV8 on a single image","doc":"","ref":"yolov8_single_image.html#showing-detected-objects"},{"type":"extras","title":"Just run the cell below to have the `YOLODraw` util. - YoloV8 on a single image","doc":"<!-- livebook:{\"reevaluate_automatically\":true} -->\n\n```elixir\n##\ndefmodule YOLODraw do\n  @font_size 18\n  @stroke_width 3\n  def draw_detected_objects(mat, detected_objects) do\n    {:ok, image} = Image.from_evision(mat)\n\n    detected_objects\n    |> Enum.reduce(image, fn %{bbox: bbox}=od, image ->\n      left = max(round(bbox.cx - bbox.w/2), 0)\n      top = max(round(bbox.cy - bbox.h/2), 0)\n      prob = round(od.prob * 100)\n      color = class_color(od.class_idx)\n      \n      text_image = \n        Image.Text.simple_text!(\"#{od.class} #{prob}%\", text_fill_color: \"white\", font_size: @font_size)\n        |> Image.Text.add_background_padding!(background_fill_color: color, padding: [5, 5])\n        |> Image.Text.add_background!(background_fill_color: color)\n        |> Image.split_alpha()\n        |> elem(0)\n      {_, text_hight, _} = Image.shape(text_image)\n      \n      image\n      |> Image.Draw.rect!(left,top,bbox.w,bbox.h,[\n        stroke_width: @stroke_width, color: color, fill: false\n      ])\n      |> Image.Draw.image!(text_image, left, max(top - text_hight - 2, 0))\n      \n    end)\n  end\n  \n  @class_colors  [\n    \"#FF0000\", \"#00FF00\", \"#0000FF\", \"#FFFF00\", \"#FF00FF\", \"#00FFFF\", \n    \"#800000\", \"#008000\", \"#000080\", \"#FF00FF\", \"#800080\", \"#008080\", \n    \"#C0C0C0\", \"#FFA500\", \"#A52A2A\", \"#8A2BE2\", \"#5F9EA0\", \"#7FFF00\", \n    \"#D2691E\", \"#FF7F50\", \"#6495ED\", \"#DC143C\", \"#00FFFF\", \"#00008B\", \n    \"#008B8B\", \"#B8860B\", \"#A9A9A9\", \"#006400\", \"#BDB76B\", \"#8B008B\", \n    \"#556B2F\", \"#FF8C00\", \"#9932CC\", \"#8B0000\", \"#E9967A\", \"#8FBC8F\", \n    \"#483D8B\", \"#2F4F4F\", \"#00CED1\", \"#9400D3\", \"#FF1493\", \"#00BFFF\", \n    \"#696969\", \"#1E90FF\", \"#B22222\", \"#FFFAF0\", \"#228B22\", \"#FF00FF\", \n    \"#DCDCDC\", \"#F8F8FF\", \"#FFD700\", \"#DAA520\", \"#808080\", \"#ADFF2F\", \n    \"#F0FFF0\", \"#FF69B4\", \"#CD5C5C\", \"#4B0082\", \"#FFFFF0\", \"#F0E68C\", \n    \"#E6E6FA\", \"#FFF0F5\", \"#7CFC00\", \"#FFFACD\", \"#ADD8E6\", \"#F08080\", \n    \"#E0FFFF\", \"#FAFAD2\", \"#D3D3D3\", \"#90EE90\", \"#FFB6C1\", \"#FFA07A\", \n    \"#20B2AA\", \"#87CEFA\", \"#778899\", \"#B0C4DE\", \"#FFFFE0\", \"#00FF7F\", \n    \"#4682B4\", \"#D2B48C\", \"#008080\", \"#D8BFD8\", \"#FF6347\", \"#40E0D0\", \n    \"#EE82EE\", \"#F5DEB3\", \"#FFFFFF\", \"#F5F5F5\"\n  ]\n  |> Enum.with_index(&{&2, &1})\n  |> Map.new()  \n\n  def class_color(class_idx) do\n    Map.get(@class_colors, class_idx, \"#FF0000\")\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, YOLODraw, <<70, 79, 82, 49, 0, 0, 22, ...>>, {:class_color, 1}}\n```\n\n```elixir\nYOLODraw.draw_detected_objects(mat, detected_objects)\n```\n\n<!-- livebook:{\"branch_parent_index\":3} -->","ref":"yolov8_single_image.html#just-run-the-cell-below-to-have-the-yolodraw-util"},{"type":"extras","title":"YoloV8x - YoloV8 on a single image","doc":"","ref":"yolov8_single_image.html#yolov8x"},{"type":"extras","title":"Use the python script to generate the `yolov8x.onnx` model! - YoloV8 on a single image","doc":"Simply run\n\n```\npython python/yolov8_to_onnx.py x\n```\n\n```elixir\nlarge_model_path = Kino.FS.file_path(\"yolov8x.onnx\")\n\nmodel_x = YOLO.load(model_path: large_model_path, classes_path: classes_path)\n\ndetected_objects =\n  model_x\n  |> YOLO.detect(mat)\n  |> YOLO.to_detected_objects(model_x.classes)    \n\n\nYOLODraw.draw_detected_objects(mat, detected_objects)\n```\n\n<!-- livebook:{\"offset\":8648,\"stamp\":{\"token\":\"XCP.ZIlRKShT74_okROe1jpch0L7-Umlyl1wztxsTQR4mv7xczQDDkEE9tQJVESFQgtTauiIi68AqamaKChuEXSAlmE-meOHNSNvBU6_-w\",\"version\":2}} -->","ref":"yolov8_single_image.html#use-the-python-script-to-generate-the-yolov8x-onnx-model"},{"type":"extras","title":"Make it fast with YoloFastNMS","doc":"<!-- livebook:{\"file_entries\":[{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/test/fixtures/traffic.jpg\"},\"name\":\"traffic.jpg\",\"type\":\"file\"},{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/models/yolov8n.onnx\"},\"name\":\"yolov8n.onnx\",\"type\":\"file\"},{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/models/yolov8n_classes.json\"},\"name\":\"yolov8n_classes.json\",\"type\":\"file\"}],\"persist_outputs\":true} -->\n\n# Make it fast with YoloFastNMS\n\n```elixir\nMix.install([\n  {:yolo, \">= 0.0.0\"},\n  {:yolo_fast_nms, \">= 0.0.0\"},\n  {:nx, \"~> 0.9.1\"},\n  {:exla, \"~> 0.9.0\"},\n  {:kino, \"~> 0.14.0\"},\n  {:evision, \"~> 0.2\"}\n],[\n  config: [\n    nx: [default_backend: EXLA.Backend]\n  ]\n])\n```","ref":"yolo_fast_nms.html"},{"type":"extras","title":"Load image and model - Make it fast with YoloFastNMS","doc":"```elixir\nmat = Evision.imread(Kino.FS.file_path(\"traffic.jpg\"))\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Evision.Mat{\n  channels: 3,\n  dims: 2,\n  type: {:u, 8},\n  raw_type: 16,\n  shape: {1080, 1920, 3},\n  ref: #Reference<0.1364157824.1534197778.893>\n}\n```\n\n```elixir\nmodel =\n  YOLO.load(\n    model_path: Kino.FS.file_path(\"yolov8n.onnx\"),\n    classes_path: Kino.FS.file_path(\"yolov8n_classes.json\")\n  )\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%YOLO.Model{\n  ref: #Ortex.Model ,\n  classes: %{\n    39 => \"bottle\",\n    74 => \"clock\",\n    59 => \"bed\",\n    69 => \"oven\",\n    67 => \"cell phone\",\n    45 => \"bowl\",\n    50 => \"broccoli\",\n    22 => \"zebra\",\n    51 => \"carrot\",\n    26 => \"handbag\",\n    63 => \"laptop\",\n    47 => \"apple\",\n    27 => \"tie\",\n    77 => \"teddy bear\",\n    0 => \"person\",\n    5 => \"bus\",\n    21 => \"bear\",\n    62 => \"tv\",\n    30 => \"skis\",\n    16 => \"dog\",\n    3 => \"motorcycle\",\n    53 => \"pizza\",\n    33 => \"kite\",\n    14 => \"bird\",\n    40 => \"wine glass\",\n    37 => \"surfboard\",\n    24 => \"backpack\",\n    17 => \"horse\",\n    48 => \"sandwich\",\n    73 => \"book\",\n    11 => \"stop sign\",\n    57 => \"couch\",\n    43 => \"knife\",\n    6 => \"train\",\n    20 => \"elephant\",\n    60 => \"dining table\",\n    28 => \"suitcase\",\n    25 => \"umbrella\",\n    1 => \"bicycle\",\n    58 => \"potted plant\",\n    32 => \"sports ball\",\n    76 => \"scissors\",\n    36 => \"skateboard\",\n    35 => \"baseball glove\",\n    15 => \"cat\",\n    78 => \"hair drier\",\n    64 => \"mouse\",\n    75 => \"vase\",\n    ...\n  },\n  model_impl: YOLO.Models.YoloV8,\n  shapes: %{input: {1, 3, 640, 640}, output: {1, 84, 8400}}\n}\n```","ref":"yolo_fast_nms.html#load-image-and-model"},{"type":"extras","title":"YOLO.NMS vs YoloFastNMS - Make it fast with YoloFastNMS","doc":"```elixir\n# YOLO.NMS\n{nx_nms_time, _result} = :timer.tc fn ->\n  YOLO.detect(model, mat)\nend\n\n# YOLO.NMS\n{fast_nms_time, _result} = :timer.tc fn ->\n  YOLO.detect(model, mat, nms_fun: &YoloFastNMS.run/3)\nend\n\nIO.puts(\"YOLO.NMS: #{trunc(nx_nms_time/1_000)}ms\")\nIO.puts(\"YoloFastNMS: #{trunc(fast_nms_time/1_000)}ms\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nYOLO.NMS: 550ms\nYoloFastNMS: 37ms\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\n<!-- livebook:{\"offset\":3259,\"stamp\":{\"token\":\"XCP.pLpS1H6T6p4gnAG5PmQJYu0ou6lAipSPxWIePkZTQU2Y5HTezxj9419oQ9tkQDkBiw1RRuQ5uQ-aQxx5xBBtx_xydnwMJYRyHDr3vQ\",\"version\":2}} -->","ref":"yolo_fast_nms.html#yolo-nms-vs-yolofastnms"},{"type":"extras","title":"Real-time Object Detection","doc":"<!-- livebook:{\"file_entries\":[{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/models/yolov8n.onnx\"},\"name\":\"yolov8n.onnx\",\"type\":\"file\"},{\"file\":{\"file_system_id\":\"local\",\"file_system_type\":\"local\",\"path\":\"/Users/alvise/code/yolo/models/yolov8n_classes.json\"},\"name\":\"yolov8n_classes.json\",\"type\":\"file\"}]} -->\n\n# Real-time Object Detection\n\n```elixir\nMix.install([\n  {:yolo, \">= 0.0.0\"},\n  {:yolo_fast_nms, \">= 0.0.0\"},\n  {:nx, \"~> 0.9.1\"},\n  {:exla, \"~> 0.9.0\"},\n  {:image, \"~> 0.54.4\"},\n  {:evision, \"~> 0.2.0\"},\n  {:kino, \"~> 0.14.0\"}\n],[\n  config: [\n    nx: [default_backend: EXLA.Backend],\n  ]\n])\n\n```","ref":"yolov8_webcam.html"},{"type":"extras","title":"Load Model - Real-time Object Detection","doc":"```elixir\nmodel_path = Kino.FS.file_path(\"yolov8n.onnx\")\nclasses_path = Kino.FS.file_path(\"yolov8n_classes.json\")\n\nmodel = YOLO.load(model_path: model_path, classes_path: classes_path)\n```","ref":"yolov8_webcam.html#load-model"},{"type":"extras","title":"Load WebCam with Evision - Real-time Object Detection","doc":"```elixir\ncamera = Evision.VideoCapture.videoCapture(0)\n```\n\n```elixir\nEvision.VideoCapture.read(camera)\n```","ref":"yolov8_webcam.html#load-webcam-with-evision"},{"type":"extras","title":"Run Object Detection and render detections on the frame - Real-time Object Detection","doc":"```elixir\ndefmodule YOLODraw do\n  @font_size 18\n  @stroke_width 3\n  def draw_detected_objects(mat, detected_objects, fps_label) do\n    {:ok, image} = Image.from_evision(mat)\n    {full_width, full_height, _} = Image.shape(image)\n    \n    fps_image = \n      Image.Text.simple_text!(fps_label, text_fill_color: \"white\", font_size: 21)\n      |> Image.Text.add_background_padding!(background_fill_color: \"#0000FF\", padding: [5, 5])\n      |> Image.Text.add_background!(background_fill_color: \"#0000FF\")\n      |> Image.split_alpha()\n      |> elem(0)\n    {fps_width, fps_height, _} = Image.shape(fps_image)\n\n    detected_objects\n    |> Enum.reduce(image, fn %{bbox: bbox}=od, image ->\n      left = max(round(bbox.cx - bbox.w/2), 0)\n      top = max(round(bbox.cy - bbox.h/2), 0)\n      prob = round(od.prob * 100)\n      color = class_color(od.class_idx)\n      \n      text_image = \n        Image.Text.simple_text!(\"#{od.class} #{prob}%\", text_fill_color: \"white\", font_size: @font_size)\n        |> Image.Text.add_background_padding!(background_fill_color: color, padding: [5, 5])\n        |> Image.Text.add_background!(background_fill_color: color)\n        |> Image.split_alpha()\n        |> elem(0)\n\n      {_, text_height, _} = Image.shape(text_image)\n      \n      image\n      |> Image.Draw.rect!(left,top,bbox.w,bbox.h,[\n        stroke_width: @stroke_width, color: color, fill: false\n      ])\n      |> Image.Draw.image!(text_image, left, max(top - text_height - 2, 0))\n    end)\n    |> Image.Draw.image!(fps_image, full_width - fps_width, full_height - fps_height)\n  end\n  \n  @class_colors  [\n    \"#FF0000\", \"#00FF00\", \"#0000FF\", \"#FFFF00\", \"#FF00FF\", \"#00FFFF\", \n    \"#800000\", \"#008000\", \"#000080\", \"#FF00FF\", \"#800080\", \"#008080\", \n    \"#C0C0C0\", \"#FFA500\", \"#A52A2A\", \"#8A2BE2\", \"#5F9EA0\", \"#7FFF00\", \n    \"#D2691E\", \"#FF7F50\", \"#6495ED\", \"#DC143C\", \"#00FFFF\", \"#00008B\", \n    \"#008B8B\", \"#B8860B\", \"#A9A9A9\", \"#006400\", \"#BDB76B\", \"#8B008B\", \n    \"#556B2F\", \"#FF8C00\", \"#9932CC\", \"#8B0000\", \"#E9967A\", \"#8FBC8F\", \n    \"#483D8B\", \"#2F4F4F\", \"#00CED1\", \"#9400D3\", \"#FF1493\", \"#00BFFF\", \n    \"#696969\", \"#1E90FF\", \"#B22222\", \"#FFFAF0\", \"#228B22\", \"#FF00FF\", \n    \"#DCDCDC\", \"#F8F8FF\", \"#FFD700\", \"#DAA520\", \"#808080\", \"#ADFF2F\", \n    \"#F0FFF0\", \"#FF69B4\", \"#CD5C5C\", \"#4B0082\", \"#FFFFF0\", \"#F0E68C\", \n    \"#E6E6FA\", \"#FFF0F5\", \"#7CFC00\", \"#FFFACD\", \"#ADD8E6\", \"#F08080\", \n    \"#E0FFFF\", \"#FAFAD2\", \"#D3D3D3\", \"#90EE90\", \"#FFB6C1\", \"#FFA07A\", \n    \"#20B2AA\", \"#87CEFA\", \"#778899\", \"#B0C4DE\", \"#FFFFE0\", \"#00FF7F\", \n    \"#4682B4\", \"#D2B48C\", \"#008080\", \"#D8BFD8\", \"#FF6347\", \"#40E0D0\", \n    \"#EE82EE\", \"#F5DEB3\", \"#FFFFFF\", \"#F5F5F5\"\n  ]\n  |> Enum.with_index(&{&2, &1})\n  |> Map.new()  \n\n  def class_color(class_idx) do\n    Map.get(@class_colors, class_idx, \"#FF0000\")\n  end\nend\n```\n\n```elixir\ndefmodule FPSCalc do\n  defstruct [\n    window: [], \n    count: 0,\n    max: 30\n  ]\n  \n  def new(max), do: %__MODULE__{max: max}\n  def avg_ms(calc) do\n    Enum.sum(calc.window)/calc.count\n  end\n\n  def fps(calc) do\n    trunc(1_000/avg_ms(calc))\n  end\n  \n  def add(calc, ms) do\n    window = calc.window ++ [ms]\n    count = Enum.count(window)\n    \n\n    if count > calc.max do\n      %{calc | window: tl(window)}\n    else\n      %{calc | window: window, count: calc.count + 1}\n    end\n  end\nend\n\nStream.repeatedly(fn -> \n  mat = Evision.VideoCapture.read(camera)\n  {microseconds, detected_objects} = :timer.tc fn ->\n    model\n    |> YOLO.detect(mat, prob_threshold: 0.5, nms_fun: &YoloFastNMS.run/3)\n    |> YOLO.to_detected_objects(model.classes)\n  end\n  \n  {microseconds/1000, mat, detected_objects}\nend)\n|> Kino.animate(FPSCalc.new(30), fn {ms, mat, detected_objects}, fps_calc ->\n  fps_calc = FPSCalc.add(fps_calc, ms)\n  avg_ms = FPSCalc.avg_ms(fps_calc)\n  fps = 1_000/avg_ms\n  fps_label = \"#{trunc(avg_ms)}ms - #{trunc(fps)} FPS\"\n  \n  {:cont, YOLODraw.draw_detected_objects(mat, detected_objects, fps_label), fps_calc}\nend)\n```\n\n<!-- livebook:{\"offset\":4971,\"stamp\":{\"token\":\"XCP.vD2ATUGM0y7OmOYeLNtaSDXc78Y3WQOPo8KHjsMgdeYot0a59CuhI-TUbPIyq-ZXY3XB6nZUSek7GzB2q_VSmIkqv1lWoit8M6VJbw\",\"version\":2}} -->","ref":"yolov8_webcam.html#run-object-detection-and-render-detections-on-the-frame"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.0"}}